import { SemanticMark } from "./semantic-mark";

import { GrammarMarkType, SIGNAL_VIEW_BOX, Factory } from "@visactor/vgrammar-core";

import { PlotMakType } from "./enums";

import { field as getFieldAccessor } from "@visactor/vgrammar-util";

export class SunburstSemanticMark extends SemanticMark {
    constructor(id) {
        super(PlotMakType.sunburst, id), Factory.getTransform("sunburst") || this._logger.error("Please add this line of code: import { registerSunburstTransforms } from 'vgrammar-hierarchy': \n        and run registerSunburstTransforms() before use sunburst chart");
    }
    setMarkType() {
        return GrammarMarkType.arc;
    }
    setDefaultDataTransform() {
        var _a;
        return [ {
            type: "sunburst",
            width: {
                signal: "viewWidth"
            },
            height: {
                signal: "viewHeight"
            },
            nodeKey: null === (_a = this.spec.encode) || void 0 === _a ? void 0 : _a.node,
            flatten: !0
        } ];
    }
    getDataIdOfLabel() {
        var _a, _b;
        return `${null !== (_b = null === (_a = this.spec.data) || void 0 === _a ? void 0 : _a.id) && void 0 !== _b ? _b : this.spec.id}-data-link`;
    }
    setMultipleData() {
        return this.spec.label ? [ {
            source: this.getDataIdOfFiltered(),
            id: this.getDataIdOfLabel(),
            transform: [ {
                type: "filter",
                callback: datum => !!datum.label
            } ]
        } ] : null;
    }
    parseScaleByEncode(channel, option) {
        return "color" === channel ? {
            type: "ordinal",
            id: this.getScaleId("color"),
            domain: {
                data: this.getDataIdOfFiltered(),
                field: option
            },
            range: this.getPalette()
        } : null;
    }
    convertMarkEncode(encode) {
        var _a, _b, _c;
        const markEncoder = this.convertSimpleMarkEncode(encode), res = {
            x: {
                field: "x"
            },
            y: {
                field: "y"
            },
            innerRadius: {
                field: "innerRadius"
            },
            outerRadius: {
                field: "outerRadius"
            },
            startAngle: {
                field: "startAngle"
            },
            endAngle: {
                field: "endAngle"
            }
        };
        if (markEncoder.stroke && (res.stroke = markEncoder.stroke), markEncoder.color) {
            const scaleColorId = this.getScaleId("color"), colorAccessor = getFieldAccessor(markEncoder.color.field);
            res.fill = (datum, el, params) => {
                const scale = params[scaleColorId];
                return (null == datum ? void 0 : datum.datum) ? scale.scale(colorAccessor(datum.datum[datum.datum.length - 1])) : void 0;
            };
        } else res.fill = null !== (_b = null === (_a = this.spec.style) || void 0 === _a ? void 0 : _a.fill) && void 0 !== _b ? _b : null === (_c = this.getPalette()) || void 0 === _c ? void 0 : _c[0];
        return res;
    }
    setMainMarkEnterEncode() {
        var _a;
        return null === (_a = this.spec.style) || void 0 === _a ? void 0 : _a.nodeStyle;
    }
    setMainMarkSpec() {
        return {
            key: "key"
        };
    }
    parseLabelSpec() {
        return {
            marks: [],
            interactions: []
        };
    }
    setMultiMarksSpec() {
        const label = this.spec.label;
        return label ? Object.keys(label).map((key => {
            const textGetter = getFieldAccessor(key);
            return {
                id: `${this.getMarkId()}-text-${key}`,
                type: "text",
                from: {
                    data: this.getDataIdOfLabel()
                },
                layout: {
                    position: "content",
                    skipBeforeLayouted: !0
                },
                key: "flattenIndex",
                dependency: this.viewSpec.scales.map((scale => scale.id)).concat(SIGNAL_VIEW_BOX),
                animation: this.convertMarkAnimation(),
                encode: {
                    enter: label[key].textStyle,
                    update: (datum, el, params) => ({
                        x: datum.label.x,
                        y: datum.label.y,
                        textAlign: datum.label.textAlign,
                        textBaseline: datum.label.textBaseline,
                        text: textGetter(datum.datum[datum.datum.length - 1]),
                        angle: datum.label.angle,
                        maxLineWidth: datum.label.maxLineWidth
                    })
                }
            };
        })) : null;
    }
}

SunburstSemanticMark.type = PlotMakType.sunburst;
//# sourceMappingURL=sunburst.js.map